// 删除二叉树根节点函数
// 要点：
//      根据被删除节点的三种情况，分别进行处理。
//      1）被删除节点同时有左子树与右子树。2）被删除节点只有左子树或只有右子树。3）被删除节点没有子树。
// 关键字：二叉树，删除节点
#include <stdlib.h>

typedef struct node{
    int data;
    struct node *l; 
    struct node *r;
}Node;
///////////////////////////////////声明结构体自引用l，r域构成形状树////////////////////////////////////
//@ shape l, r : tree;

////////////////////////////////////////////声明逻辑变量/////////////////////////////////////////////
//@ logic Node* oldp; //oldp用于记录函数入口处的指针p


//////////////////////////////////////TREEdelete函数的验证//////////////////////////////////////////
//函数协议：
//  前条件：描述函数参数是一棵二叉树且不为空；逻辑指针变量记录函数入口处被删除的节点
//  后条件：描述函数的返回值是一棵树，且实际删除的是根节点（情况2）和3）），或根节点的前驱节点（情况1））
/*@
  requires 
      \tree(p) && p != \null && oldp == p 
      ;
  assigns oldp;
  ensures 
      \tree(\result) &&  (\dangling(oldp) || oldp == \result)
      //返回值是二叉树 &&  删除的是(根节点   || 不是根节点（根节点的前驱节点））
      ;
*/
Node * TREEdelete(Node *p) {
    Node *q; Node *s;

    q = p;
    if(p->r==NULL){  // 右子树为空
        p = p->l;
        free(q);
    } else if(p->l==NULL){ // 左子树为空
        p = p->r;
        free(q);
    } else { // 左右子树不为空，寻找前驱节点，将前驱节点拷贝到根节点，删除前驱节点
        s = p->l;
        if(s->r == NULL){
            p->l = s->l;
            p->data = s->data;
            free(s);
        } else {
            q = s;
            s = s->r;
            /*@ 
              loop invariant
                  oldp == p 
                  && \tree(p->r) && p->r != \null // p指向节点的右子节点构成树 ，且该右子节点不为空
                  && \tree_seg_r(p->l, q) //
                  && \tree(q->l) 
                  && s == q->r 
                  && \tree(s)
                  ;
            */
            while(s->r != NULL){
                q = s;
                s = s->r;
            }
            p->data = s->data;
            q->r = s->l;
            free(s);
        }
    }
    return p;
}